@@ -23,6 +23,8 @@ from subprocess import Popen
 from os import path
 from itertools import *
 from operator import itemgetter
+from calculate.install.distr import FlashDistributive, DistributiveError, \
+    IsoDistributive
 from calculate.lib.datavars import (TableVariable, Variable, VariableError,
                                     ReadonlyVariable, ReadonlyTableVariable,
                                     SourceReadonlyVariable, VariableInterface,
@@ -35,10 +37,11 @@ from calculate.lib.utils.device import (getUdevDeviceInfo, getDeviceType,
                                         getUUIDDict, getCommonDeviceName)
 from calculate.install.variables.autopartition import Sizes
 from calculate.lib.utils.files import (listDirectory, pathJoin, readFile, FStab,
-                                       isMount, getProgPath)
+                                       isMount, getProgPath, DiskSpace)
 from calculate.install.fs_manager import FileSystemManager
 
 from calculate.lib.cl_lang import setLocalTranslate, _
+from calculate.lib.variables.system import RootType
 
 setLocalTranslate('cl_install3', sys.modules[__name__])
 
@@ -632,7 +635,7 @@ class VariableOsDiskPart(ReadonlyVariable):
         return \
             map(lambda x: x[0] if x[1] else "",
                 map(lambda x: (
-                getPartitionType(getUdevDeviceInfo(name=x[0])), x[1]),
+                    getPartitionType(getUdevDeviceInfo(name=x[0])), x[1]),
                     self.ZipVars('os_disk_dev', 'os_disk_parent')))
 
 
@@ -971,7 +974,8 @@ class VariableOsLocationDest(LocationHelper, Variable):
         ################################
         # check size for root device
         ################################
-        if self.Get('os_install_root_type') != "flash" and \
+        osInstallRootType = self.Get('os_install_root_type')
+        if osInstallRootType != "flash" and \
                 not "/usr" in value:
             for mp, size in filter(lambda x: x[0] == '/' and x[1].isdigit() and \
                             int(x[1]) < 7 * 1024 * 1024 * 1024,
@@ -1174,8 +1178,8 @@ class VariableOsLocationFormat(LocationHelper, Variable):
             # check compatible fs for mount point only root dirs
             if dev.startswith('/dev/') and mp and (mp.count('/') == 1 or
                                                            mp in (
-                                                       '/var/calculate',
-                                                       '/boot/efi')):
+                                                               '/var/calculate',
+                                                               '/boot/efi')):
                 if not FileSystemManager.checkFSForTypeMount(fs,
                                                              osInstallRootType,
                                                              mp):
@@ -1271,7 +1275,7 @@ class VariableOsLocationPerformFormat(LocationHelper, Variable):
                     raise VariableError(
                         _(
                             "{device} must but cannot be formatted, as it is mounted to {mountpoint} on the current system"
-                            ).format(
+                        ).format(
                             device=dev, mountpoint=diskMount.get(dev, '')))
                 elif isMount(dev):
                     raise VariableError(
@@ -1470,6 +1474,11 @@ class VariableOsInstallDiskDevBase(DeviceHelper, ReadonlyVariable):
     type = "list"
 
     def get(self):
+        if self.Get('cl_install_type') == 'flash':
+            disk = self.Get('os_install_disk_single')
+            if disk:
+                return [getCommonDeviceName(disk)]
+            return []
         return map(getCommonDeviceName,
                    map(lambda x: x[0],
                        filter(lambda x: x[0].startswith('/dev/') and x[1],
@@ -1532,6 +1541,11 @@ class VariableOsInstallDiskMountBase(ReadonlyVariable):
     type = "list"
 
     def get(self):
+        if self.Get('cl_install_type') == 'flash':
+            disk = self.Get('os_install_disk_single')
+            if disk:
+                return ["/"]
+            return []
         return map(lambda x: x[1],
                    filter(lambda x: x[0].startswith('/dev/') and x[1],
                           zip(self.Get('os_location_source'),
@@ -1585,10 +1599,10 @@ class VariableOsInstallDiskOptions(ReadonlyVariable):
         return map(postProcess,
                    map(lambda x: diskOpts.get(x[0], x[1]),
                        map(lambda x: (
-                       x[0], FileSystemManager.getDefaultOpt(x[1],
-                                                             ssd=ssdDisk.get(
-                                                                 x[2],
-                                                                 'off') == 'on')),
+                           x[0], FileSystemManager.getDefaultOpt(x[1],
+                                                                 ssd=ssdDisk.get(
+                                                                     x[2],
+                                                                     'off') == 'on')),
                            self.ZipVars('os_install_disk_dev',
                                         'os_install_disk_format',
                                         'os_install_disk_parent'))))
@@ -2015,17 +2029,20 @@ class VariableOsInstallRootType(LocationHelper, Variable):
         self.label = _("Installation type")
 
     def get(self):
-        if self.Get('cl_autopartition_set') == 'on':
+        selectRootType = self.Get('cl_install_type')
+        if not selectRootType:
+            return self.Get('os_root_type')
+        if selectRootType == "flash":
+            return "flash"
+        else:
+            rootdev = self.Get('os_install_root_dev')
+            device = getPartitionDevice(
+                getUdevDeviceInfo(name=rootdev).get('DEVPATH', ''))
+            device_type = self.Select(
+                'os_device_type', where='os_device_dev', eq=device, limit=1)
+            if device_type in ("usb-hdd", "flash"):
+                return "usb-hdd"
             return "hdd"
-        rootdev = self.Get('os_install_root_dev')
-        devicetype = getPartitionDevice(
-            getUdevDeviceInfo(name=rootdev).get('DEVPATH', ''))
-        currentRootType = self.Get('os_root_type')
-        if self.Get('cl_action') == 'system':
-            currentRootType = "hdd"
-        return self.Select('os_device_type', where='os_device_dev',
-                           eq=devicetype, limit=1) \
-               or currentRootType
 
     def choice(self):
         return [("hdd", _("Hard disk")),
@@ -2088,8 +2105,9 @@ class VariableOsInstallFstabMountConf(DeviceHelper, ReadonlyVariable):
         # rootLine one string, but it correct work if devicesForFstab is empty
         rootLine = "\n".join(map(lambda x: "%s\t%s\t%s\t%s\t0 1" %
                                            (
-                                           self._commentFstab(x[0], x[1], x[4]),
-                                           x[1], x[2], x[3]),
+                                               self._commentFstab(x[0], x[1],
+                                                                  x[4]),
+                                               x[1], x[2], x[3]),
                                  devicesForFstab[:1]))
         otherLines = "\n".join(map(lambda x: "%s\t%s\t%s\t%s\t0 0" %
                                              (self._commentFstab(x[0], x[1],
@@ -2123,56 +2141,124 @@ class VariableOsInstallFstabSwapConf(VariableOsInstallFstabMountConf):
 class VariableClInstallType(Variable):
     """
     Installation type (extension variable describe, that
-    install must be to flash
+    install must be to flash or hdd
     """
     type = "choice"
     value = ""
 
     def choice(self):
-        return ["", "flash"]
-
-    def check(self, value):
-        if value == "flash" and \
-                        len(self.Choice('os_install_disk_single')) == 1:
-            raise VariableError(_("Flash drive not found") + '.')
+        return ["", "flash", "hdd"]
 
 
 class VariableOsInstallDiskSingle(Variable):
     """
     Installation disk
     """
-    type = "choice"
+    type = "choiceedit"
+    opt = ["--disk", "-d"]
+    metavalue = 'DISK'
+    untrusted = True
+    value = ""
 
     def init(self):
         self.label = _("Installation disk")
-
-    def availDevs(self):
-        """
-        Available devices
-        """
-        if self.Get('cl_install_type') == 'flash':
-            flashDrives = self.Select('os_device_dev',
-                                      where='os_device_type',
-                                      eq="flash")
-            return self.Select('os_disk_dev',
-                               where='os_disk_parent',
-                               _in=flashDrives)
-        else:
-            return self.Get('os_disk_dev') + self.Get('os_bind_path')
-
-    def get(self):
-        if self.Get('cl_install_type') == 'flash':
-            disks = self.availDevs()
-            if disks:
-                return disks[0]
-        return ""
+        self.help = _("set the USB Flash device")
 
     def choice(self):
         diskParentMap = dict(zip(self.Get('os_disk_dev'),
                                  self.Get('os_disk_parent')))
         deviceParentMap = dict(self.ZipVars('os_device_dev', 'os_device_name'))
+        disks = self.select('os_disk_dev', os_disk_part__ne="")
         return map(lambda x: (x, "%s (%s)" % (x,
                                               deviceParentMap.get(
                                                   diskParentMap.get(x, x),
                                                   _("Unknown")))),
-                   self.availDevs()) + [("", "")]
+                   disks)
+
+    def check(self, value):
+        # проверить, чтобы был выбран именно раздел
+        if value not in self.Get('os_disk_dev'):
+            raise VariableError(
+                _("Wrong device '%s'" % value)
+            )
+        # проверить, чтобы раздел не использовался системой (не описан в fstab)
+        mp = self.select('os_disk_mount', os_disk_dev=value, limit=1)
+        if mp:
+            raise VariableError(
+                _("The partition {dev} is already in use as {mp}").format(
+                    dev=value, mp=mp))
+        # если система загружена с флешки (не iso) - нельзя переустановить
+        # эту систему
+        root_type = self.Get('os_root_type_ext')
+        if root_type in RootType.LiveFlash:
+            if value == self.Get('os_root_flash_dev'):
+                raise VariableError(
+                    _("You cannot install the new system instead current"))
+        # detect using extended partition
+        disk_part = self.select('os_disk_part', os_disk_dev=value, limit=1)
+        if disk_part == 'extended':
+            raise VariableError(
+                _("Unable to use extended partition %s for installation") %
+                value)
+        if "cdrom" in disk_part:
+            raise VariableError(_("Unable to use CDROM %s for installation") %
+                                value)
+        if disk_part == 'gpt':
+            raise VariableError(_("You need a disk with a dos "
+                                  "table for Flash install"))
+
+
+class VariableOsInstallFormatSingleSet(Variable):
+    """
+    Форматировать Flash
+    """
+    type = "bool"
+    opt = ["--format"]
+    untrusted = True
+    value = "off"
+
+    def init(self):
+        self.label = _("Format the USB Flash")
+        self.help = _("perform the formatting of the USB Flash drive")
+
+    def must_be_formatted(self, dev):
+        fs = self.select('os_disk_format', os_disk_dev=dev, limit=1)
+        if fs != "vfat":
+            return True
+        return False
+
+    def cannot_be_formatted(self, dev):
+        flash_dev = self.Get('os_root_flash_dev')
+        return flash_dev and dev == flash_dev
+
+    def check(self, value):
+        devs = self.Get('os_disk_dev')
+        dev = self.Get('os_install_disk_single')
+        if dev not in devs:
+            return
+        if value == "on":
+            if self.cannot_be_formatted(dev):
+                raise VariableError(
+                    _("You cannot format the USB Flash which "
+                      "contains the current system"))
+        else:
+            if self.must_be_formatted(dev):
+                raise VariableError(
+                    _("{device} must be formatted").format(device=dev))
+            if dev:
+                try:
+                    with FlashDistributive(dev) as f:
+                        dn = f.getDirectory()
+                        df = DiskSpace()
+                        free_size = df.get_free(dev)
+                        squash_fn = path.join(dn, "livecd.squashfs")
+                        if not path.exists(squash_fn):
+                            source = self.Get('cl_image')
+                            if isinstance(source, IsoDistributive):
+                                image_size = source.get_squash_size()
+                                if image_size > free_size:
+                                    raise VariableError(
+                                        _("Not enough free space on the "
+                                          "USB Flash"))
+                except DistributiveError:
+                    pass
