@@ -33,7 +33,7 @@ from operator import itemgetter
 
 from calculate.lib.utils.files import readLinesFile
 from calculate.lib.utils.common import (getKernelUid, getTupleVersion,
-                                        getValueFromCmdLine)
+                                        getValueFromCmdLine, CmdlineParams)
 from itertools import *
 from calculate.install.distr import DistributiveError
 
@@ -74,7 +74,8 @@ class VariableOsInstallKernelScheduler(Variable):
         if self.Get('os_root_type') == 'livecd':
             return self.get_default()
         else:
-            currentScheduler = getValueFromCmdLine('elevator', 0)
+            currentScheduler = getValueFromCmdLine(
+                CmdlineParams.IOScheduler)
             if currentScheduler in map(lambda x: x[0], self.choice()):
                 return currentScheduler
         return self.Get('os_install_kernel_schedule_default')
